const { userInfo } = require("os");
const dotenv = require("dotenv");
const withSvgr = require("@newhighsco/next-plugin-svgr");
const crypto = require("crypto");

const dotenvResult = dotenv.config();

if (dotenvResult.error) {
  throw dotenvResult.error;
}

if (dotenvResult.parsed["JWT_SECRET"].length < 32) {
  throw new Error("JWT_SECRET has to be 32 characters or longer");
}

const ContentSecurityPolicy = `
  default-src data: 'self';
  media-src data: 'self';
  script-src 'self';
  child-src 'self';
  style-src 'unsafe-inline' 'self';
  font-src 'self';
  frame-ancestors 'self'
`;

const securityHeaders = [
  {
    key: "Content-Security-Policy",
    value: ContentSecurityPolicy.replace(/\s{2,}/g, " ").trim(),
  },
  {
    key: "Referrer-Policy",
    value: "no-referrer",
  },
  {
    key: "Permissions-Policy",
    value: "camera=(), microphone=()",
  },
  {
    key: "X-Frame-Options",
    value: "SAMEORIGIN",
  },
];

const keyPair = crypto.generateKeyPairSync("rsa", {
  modulusLength: 2048,
  publicKeyEncoding: { type: "spki", format: "pem" },
  privateKeyEncoding: { type: "pkcs1", format: "pem" },
});

/**
 * @type {import("next").NextConfig}
 **/
const nextConfig = withSvgr({
  poweredByHeader: false,
  reactStrictMode: true,
  productionBrowserSourceMaps: false,
  serverRuntimeConfig: {
    encryptionPrivateKey: keyPair.privateKey,
    JWT_SECRET: dotenvResult.parsed["JWT_SECRET"],
    VERSION_CHECK: dotenvResult.parsed["VERSION_CHECK"] || "production",
    COOKIE_MAX_AGE: Number.parseInt(dotenvResult.parsed["COOKIE_MAX_AGE"], 10),
  },
  webpack: (config, { isServer }) => {
    if (isServer) {
      config.output.publicPath = "";
    }
    return { ...config, experiments: { asyncWebAssembly: true, layers: true } };
  },
  publicRuntimeConfig: {
    encryptionPublicKey: keyPair.publicKey,
    username: userInfo({ encoding: "utf8" }).username,
  },
  async redirects() {
    return [
      {
        source: "/",
        destination: "/dashboard",
        permanent: false,
      },
      {
        source: "/push-tx",
        destination: "/tools/push-tx",
        permanent: false,
      },
    ];
  },
  async rewrites() {
    return [
      {
        source: "/logs",
        destination: "/logs/bitcoind",
      },
    ];
  },
  async headers() {
    if (process.env.NODE_ENV === "development") return [];

    return [
      {
        // Apply these headers to all routes in your application.
        source: "/:path*",
        headers: securityHeaders,
      },
    ];
  },
});

module.exports = nextConfig;
